@@ -1,13 +1,13 @@
-# Copyrights 2013-2014 by [Mark Overmeer].
+# Copyrights 2013-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 use warnings;
 use strict;
 
 package Any::Daemon::HTTP::Proxy;
 use vars '$VERSION';
-$VERSION = '0.25';
+$VERSION = '0.26';
 
 use parent 'Any::Daemon::HTTP::Source';
 
@@ -15,6 +15,7 @@ use Log::Report    'any-daemon-http';
 
 use LWP::UserAgent ();
 use HTTP::Status   qw(HTTP_TOO_MANY_REQUESTS);
+use Time::HiRes    qw(time);
 
 
 sub init($)
@@ -33,13 +34,16 @@ sub init($)
     {   $self->{ADHDP_proxy} = ref $rem eq 'CODE' ? $rem : sub {$rem};
     }
 
+    $self->{ADHDP_fwd_to}    = $args->{forward_timeout} // 100;
+
+    # to be run before a request can be sent off
     my @prepare  =
       ( $self->stripHeaders($args->{strip_req_headers})
       , $self->addHeaders  ($args->{add_req_headers})
       , $args->{change_request} || ()
       );
-    
 
+    # to be run before a response is passed on the the client
     my @postproc =
       ( $self->stripHeaders($args->{strip_resp_headers})
       , $self->addHeaders  ($args->{add_resp_headers})
@@ -65,16 +69,19 @@ sub remoteProxy(@)
 
 #-----------------
 
+my $last_used_proxy = '';
 sub _collect($$$$)
 {   my ($self, $vhost, $session, $req, $rel_uri) = @_;
+    my $resp;
 
-    my $tohost = $req->header('Host') || $vhost->name;
+    my $vhost_name = $vhost ? $vhost->name : '';
+    my $tohost = $req->header('Host') || $vhost_name;
 
     #XXX MO: need to support https as well
     my $uri    = URI->new_abs($rel_uri, "http://$tohost");
 
     # Via: RFC2616 section 14.45
-    my $my_via = '1.1 ' . ($self->via // $uri->host_port);
+    my $my_via = '1.1 ' . ($self->via // $vhost_name);
     if(my $via = $req->header('Via'))
     {   foreach (split /\,\s+/, $via)
         {   return HTTP::Response->new(HTTP_TOO_MANY_REQUESTS)
@@ -86,22 +93,60 @@ sub _collect($$$$)
     {   $req->push_header(Via => $my_via);
     }
 
-    $self->$_($req, $uri) for @{$self->{ADHDP_prepare}};
+    $self->$_($req, $uri)
+        for @{$self->{ADHDP_prepare}};
 
-    my $ua   = $self->userAgent;
-    $req->uri($uri);
-    if(my $proxy = $self->remoteProxy($session, $req))
+    my $ua      = $self->userAgent;
+    my @proxies = grep defined, $self->remoteProxy(HTTP => $session,$req,$uri);
+
+    if(@proxies)
     {   $self->proxify($req, $uri);
-        $ua->proxy($uri->scheme, $proxy);
+        if($proxies[0] ne $last_used_proxy)
+        {   # put last_used_proxy as first try.  UserAgent reuses connection
+            @proxies = ($last_used_proxy, grep $_ ne $last_used_proxy, @proxies)
+                if grep $_ eq $last_used_proxy, @proxies;
+        }
+
+        my $start   = time;
+        my $timeout = 3;
+        while(time - $start < $self->{ADHDP_fwd_to})
+        {   my $proxy = shift @proxies;
+
+            # redirect to next proxy
+            $ua->proxy($uri->scheme, $proxy)
+                if $proxy ne $last_used_proxy;
+
+            $last_used_proxy = $proxy;
+            $ua->timeout($timeout);
+
+            my $start_req = time;
+            $resp = $ua->request($req);
+
+            info __x"request {method} {uri} via {proxy}: {status} in {t%d}ms"
+              , method => $req->method, uri => "$uri", proxy => $proxy
+              , status => $resp->code, t => (time-$start_req)*1000;
+
+            last unless $resp->is_error;
+
+            $timeout++;  # each attempt waits one second longer
+
+            # rotate attempted proxies
+            push @proxies, $proxy;
+        }
     }
     else
     {   $ua->proxy($uri->scheme, undef);
+        $last_used_proxy = '';
+
+        $ua->timeout(180);
+        $resp = $ua->request($req);
+        info __x"request {method} {uri} without proxy: {status}"
+          , method => $req->method, uri => "$uri", status => $resp->code;
     }
 
-    info __x"request {method} {uri}", method => $req->method, uri => "$uri";
-    my $resp = $ua->request($req);
+    $self->$_($resp, $uri)
+        for @{$self->{ADHDP_postproc}};
 
-    $self->$_($resp, $uri) for @{$self->{ADHDP_postproc}};
     $resp;
 }
 
