@@ -1,13 +1,13 @@
-# Copyrights 2013-2014 by [Mark Overmeer].
+# Copyrights 2013-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 use warnings;
 use strict;
 
 package Any::Daemon::HTTP::VirtualHost;
 use vars '$VERSION';
-$VERSION = '0.25';
+$VERSION = '0.26';
 
 use Log::Report    'any-daemon-http';
 
@@ -37,7 +37,11 @@ sub init($)
         or error __x"virtual host {pkg} has no name", pkg => ref $self;
 
     my $aliases = $args->{aliases} || [];
-    $self->{ADHV_aliases}  = ref $aliases eq 'ARRAY' ? $aliases : [$aliases];
+    $self->{ADHV_aliases}
+      = ref $aliases eq 'ARRAY' ? $aliases
+      : $aliases eq 'AUTO'      ? [ $self->generateAliases($name) ]
+      :                           [ $aliases ];
+
     $self->{ADHV_handlers} = $args->{handler} || $args->{handlers} || {};
     $self->{ADHV_rewrite}  = $self->_rewrite_call($args->{rewrite});
     $self->{ADHV_redirect} = $self->_redirect_call($args->{redirect});
@@ -90,6 +94,17 @@ sub _auto_docs($)
 sub name()    {shift->{ADHV_name}}
 sub aliases() {@{shift->{ADHV_aliases}}}
 
+
+sub generateAliases($)
+{   my ($thing, $h) = @_;
+    my @a;
+    $h    =~ m/^(([^.:]+)(?:[^:]*)?)(?:\:([0-9]+))?$/;
+    push @a, $1      if $3;              # name with port
+    push @a, $2      if $1 ne $2;        # hostname vs fqdn
+    push @a, "$2:$3" if $1 ne $2 && $3;  # hostname with port
+    @a;
+}
+
 #---------------------
 
 sub addHandler(@)
@@ -138,10 +153,6 @@ sub findHandler(@)
     sub { HTTP::Response->new(HTTP_NOT_FOUND) };
 }
 
-#-----------------
-
-
-#-----------------
 
 sub handleRequest($$$;$)
 {   my ($self, $server, $session, $req, $uri) = @_;
